
#ifndef _GDFONTS_H_
#define _GDFONTS_H_ 1

#ifdef __cplusplus
extern "C" {
#endif

/*
	This is a header file for gd font, generated using
	bdftogd version 0.51 by Jan Pazdziora, adelton@fi.muni.cz
	from bdf font
	-Cronyx-Courier-Medium-R-Normal--10-100-75-75-M-60-KOI8-R
	at Fri Aug 14 16:21:53 1998.
	The original bdf was holding following copyright:
	"Copyright (C) 1990, 1991 EWT Consulting, Portions Copyright (C) 1996-1997 by Andrey A. Chernov, Moscow, Russia."
 */


#include "gd.h"

extern gdFontPtr gdFontSmall;

#ifdef __cplusplus
}
#endif

#endif

